import mistune

class md_to_ascii(mistune.Renderer):

    # Pagelayout
    def block_code(self, code, lang):
        return '\n\033[92m%s\033[0m\n' % code.strip()
    def header(self, text, level, raw=None):
        if level == 2 or level == 1:
            head = '\n\033[4m\033[1m\033[37m%s\033[0m\n' % text.strip()
        else:
            head = '\n\033[1m\033[37m%s\033[0m\n' % text.strip()
        return head
    def block_quote(self, text):
        return '\n%s\n' % text.strip()
    def block_html(self, html):
        return '\n%s\n' % html.strip()
    def hrule(self):
        return '---'
    def list(self, body, ordered=True):
        return '\n%s' % body
    def list_item(self, text):
        return '* %s \n'  %  text.strip()
    def paragraph(self, text):
        return '\n%s\n' % text.strip()
    def table(self, header, body):
        return '\n%s\n' % body.strip()
    def table_row(self, content):
        return '\n%s\n' % content.strip()
    def table_cell(self, content, **flags):
        return '\n%s\n' % content.strip()

    # Inline Tags
    def double_emphasis(self, text):
        return '%s' % text.strip()
    def emphasis(self, text):
        return '%s' % text.strip()
    def codespan(self, text):
        return '\n%s\n' % text.strip()
    def linebreak(self):
        return '\n'
    def strikethrough(self, text):
        return '%s' % content.strip()
    def text(self, text):
        return '%s' % text.strip()
    def autolink(self, link, is_email=False):
        return '%s' % link.strip()
    def link(self, link, title, text):
        return '%s (%s)' % (text.strip(), link.strip())
    def image(self, src, title, text):
        return '%s' % src.strip()
    def inline_html(self, html):
        return '%s' % html.strip()
    def newline(self):
        return '\n'
    def footnote_ref(self, key, index):
        return '%s, [%s]' % (key.strip(), index.strip())
    def footnote_item(self, key, text):
        return '%s, [%s]' % (text.strip(), key.strip())
    def footnotes(self, text):
        return '%s' % text.strip()



import os
import re
import git
import socket
import smtplib
import mistune
import tempfile
import subprocess
from rendering import md_to_ascii
from email.mime.text import MIMEText

# Function definitions
def list_articles(dir):
    "lists all articles in current dir and below"
    try:
        subprocess.call(["tree", dir])
    except OSError:
        print("Error: tree not installed")
        return False

def list_directories(dir):
    "lists all directories in current dir and below"
    try:
        subprocess.call(["tree", "-d", dir])
    except OSError:
        print("Error: tree not installed")
        return False

def change_directory(dir,datadir):
    "changes directory"
    d = os.path.join(os.getcwd(),dir)

    # dont cd out of datadir
    if not datadir in d:
        d = datadir

    # if empty, switch to datadir
    if not dir:
        d = datadir

    # switch to dir
    try:
        os.chdir(d)
        return d
    except OSError:
        print("Error: Directory %s not found" % dir)

def edit_article(article, directory, editor, repo, default_commit_msg):
    """edit an article within your docs"""
    # set paths
    a = os.path.join(directory, article)
    d = os.path.dirname(a)

    # create dir(s)
    if not os.path.isdir(d):
        try:
            os.makedirs(d)
        except OSError:
            print("Error: Creation of path %s is not possible" % d)
            return False

    # start editor
    try:
        subprocess.call([editor, a])
    except OSError:
        print("Error: '%s' No such file or directory" % editor)
        return False

    # commit into git
    try:
        repo.git.add(a)
        if repo.is_dirty():
            msg = raw_input("Commit message: ")
            if not msg:
                msg = default_commit_msg
            repo.git.commit(m=msg)
        else:
            print("Nothing to commit")
    except:
        pass

def mail_article(article,dir,mailfrom):
    "mail an article to a friend"
    a = os.path.join(dir,article)

    # Create a text/plain message
    fp = open(a, 'r')
    msg = MIMEText(fp.read())
    fp.close()

    mailto = raw_input("Recipient: ")
    msg['Subject'] = article
    msg['From'] = mailfrom
    msg['To'] = mailto

    # Send the message via our own SMTP server, but don't include the
    # envelope header.
    s = smtplib.SMTP()
    try:
        s.connect()
        s.sendmail(mailfrom, [mailto], msg.as_string())
        s.quit()
    except socket.error:
        print("Error: Apparently no mailer running on your system.")
        print("Error: Could not connect to localhost:25")
        return False
    except smtplib.SMTPRecipientsRefused:
        print("Error: Invalid recipient or sender")
        return False



def view_article(article,dir,pager):
    "view an article within your docs"
    a = os.path.join(dir,article)
    # read original file
    try:
        article = open(a, "r")
    except IOError:
        print("Error: Could not find %s" % article)
        return False

    content = article.read()
    article.close()

    # hand everything over to mistune lexer
    with tempfile.NamedTemporaryFile(delete=False) as tmp:
        md = mistune.Markdown(renderer=md_to_ascii())
        tmp.write(md.render(content))

    # start pager and cleanup tmp file afterwards
    # -fr is needed for showing binary+ansi colored files to
    # be properly displayed
    try:
        subprocess.call([pager, "-fr", tmp.name])
    except OSError:
        print("Error: '%s' No such file or directory" % pager)

    try:
        os.remove(tmp.name)
    except OSError:
        print("Error: Could not remove %s" % tmp.name)

def delete_article(article,dir,repo):
    "delete an article"
    a = os.path.join(dir,article)
    try:
        repo.git.rm(a)
        repo.git.commit(m="%s deleted" % article)
        print("%s deleted" % article)
    except:
        if os.path.isdir(a):
            try:
                os.rmdir(a)
                print("Removed directory %s which was not under version control" % a)
            except OSError:
                print("Could not remove %s - its maybe not empty" % a)
        else:
            try:
                os.remove(a)
                print("Removed file %s which was not under version control" % a)
            except OSError:
                print("File %s could not be removed" % a)
    return

def move_article(dir,args,repo):
    "move an article from source to destination"
    args = args.split()
    if len(args)!=2:
        print("Invalid usage\nUse: mv source dest")
        return False

    a = os.path.join(dir,args[0])
    e = os.path.join(dir,args[1])
    d = os.path.dirname(e)

    # create dir(s)
    if not os.path.isdir(d):
        os.makedirs(d)

    # move file in git and commit
    repo.git.mv(a,e)
    repo.git.commit(m="Moved %s to %s" % (a,e))
    print("Moved %s to %s" % (a,e))

def search_article(keyword, directory, datadir, exclude):
    """
    Search for a keyword in every article within your current directory and
    below. Much like recursive grep.
    """
    c = 0
    r = re.compile(keyword)
    print("Articles:")
    for dirpath, dirs, files in os.walk(directory):
        dirs[:] = [d for d in dirs if d not in exclude]
        for fname in files:
            path = os.path.join(dirpath, fname)
            if r.search(path) is not None:
                print("* \033[92m%s\033[39m" %
                        os.path.relpath(path,datadir))
                c = c + 1
    print("Content:")
    for dirpath, dirs, files in os.walk(directory):
        dirs[:] = [d for d in dirs if d not in exclude]
        for fname in files:
            path = os.path.join(dirpath, fname)
            f = open(path, "rt")
            for i, line in enumerate(f):
                if r.search(line):
                    c = c + 1
                    print("* \033[92m%s\033[39m: %s" % (os.path.relpath(path, datadir),
                            line.rstrip('\n')))
    return "Results: %s" % c

def show_diff(args,repo):
    """
    Shows diffs for files or whole article directory
    """
    colorization = "always"
    unifiedopt = "0"

    args = args.split()
    if len(args) > 1:
        if os.path.isfile(os.path.join(os.getcwd(), args[1])):
            try:
                print(repo.git.diff('HEAD~'+args[0],args[1],
                        unified=unifiedopt, color=colorization))
            except git.exc.GitCommandError:
                print("Error: Not a valid git commit reference")
        else:
            print("Error: Wrong Usage. See help diff")
    elif len(args) == 1:
        try:
            print(repo.git.diff('HEAD~'+args[0],
                        unified=unifiedopt, color=colorization))
        except git.exc.GitCommandError:
            print("Error: Not a valid git commit reference")
    else:
        print(repo.git.diff('HEAD~5', unified="0", color="always"))

def show_log(args,repo):
    """
    Show latest git logs with specified number of entries and maybe for a
    specific file.
    """
    args = args.split()
    format="format:%C(blue)%h %Cgreen%C(bold)%ad %Creset%s"
    dateformat="short"

    if len(args) >= 1:
        if os.path.isfile(os.path.join(os.getcwd(), args[0])):
            file = args[0]
            try:
                count = args[1]
                print("Last %s commits for %s" % (count, file))
                print(repo.git.log(file, pretty=format, n=count,
                    date=dateformat))
            except IndexError:
                count = 10
                print("Last %s commits for %s" % (count, file))
                print(repo.git.log(file, pretty=format, n=count,
                        date=dateformat))
        else:
            count = args[0]
            try:
                file = args[1]
                print("Last %s commits for %s" % (count, file))
                print(repo.git.log(file, pretty=format, n=count,
                        date=dateformat))
            except IndexError:
                print("Last %s commits" % count)
                print(repo.git.log(pretty=format, n=count,
                    date=dateformat))

    elif len(args) == 0:
        count = 10
        print("Last %s commits" % count)
        try:
            print(repo.git.log(pretty=format, n=count,date=dateformat))
        except git.exc.GitCommandError:
            print("Error: git may not be configured on your system.")

def undo_change(args,repo):
    """
    You can revert your changes (use revert from git)

    Usage:
    > undo HEAD
    > undo 355f375

    Will ask for confirmation.
    """
    args = args.split()
    if len(args) == 1:
        try:
            print(repo.git.show(args[0], '--oneline', '--patience'))
            msg = raw_input("\nDo you really want to undo this? (y/n): ")
            if msg == "y":
                repo.git.revert(args[0], '--no-edit')

        except git.exc.GitCommandError:
            print("Error: Could not find given commit reference")





import os

def path_complete(self, text, line, begidx, endidx):
    """
    Path completition function used in various places for tab completion
    when using cmd
    """
    arg = line.split()[1:]

    if not arg:
        completions = os.listdir('./')
        completions[:] = [d for d in completions if d not in self.exclude]
    else:
        dir, part, base = arg[-1].rpartition('/')
        if part == '':
            dir = './'
        elif dir == '':
            dir = '/'

        completions = []
        for f in os.listdir(dir):
            if f.startswith(base):
                if os.path.isfile(os.path.join(dir,f)):
                    completions.append(f)
                else:
                    completions.append(f+'/')
    return completions



# Always prefer setuptools over distutils
from setuptools import setup, find_packages
# To use a consistent encoding
from codecs import open
import os

def read_from_file(path):
    if os.path.exists(path):
        with open(path) as input:
            return input.read()

setup(
    name='cmddocs',

    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # https://packaging.python.org/en/latest/single_source_version.html
    version='0.10.7',

    description='An interactive commandline interface for your personal docs using python, Cmd, git and markdown',
    long_description=read_from_file('README.rst'),

    # The project's main homepage.
    url='https://github.com/noqqe/cmddocs',

    # Author details
    author='Florian Baumann',
    author_email='flo@noqqe.de',

    # Choose your license
    license='MIT',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        'Development Status :: 4 - Beta',
        'Topic :: Utilities',
        'Topic :: Terminals',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2.7',
    ],

    keywords='markdown wiki commandline git',
    packages=find_packages(),
    zip_safe=True,
    install_requires=['gitpython', 'configparser', 'mistune'],

    entry_points={
        'console_scripts': [
            'cmddocs=cmddocs:main',
        ],
    },
)



#!/usr/bin/env python

import os
import cmd
import git
import sys
import signal
import ConfigParser
import pkg_resources
from os.path import expanduser
from articles import *
from completions import *

class Cmddocs(cmd.Cmd):
    """ Basic commandline interface class """

    def __init__(self):
        cmd.Cmd.__init__(self)
        self.reset = '\033[0m'
        self.read_config(self)
        self.initialize_docs(self)
        self.prompt = '\033[1m\033[' + self.promptcol + 'm' + self.prompt + " " + self.reset

    def read_config(self, conf):
        config = ConfigParser.ConfigParser()
        if not config.read(expanduser("~/.cmddocsrc")):
            print("Error: your ~/.cmddocsrc could not be read")
            exit(1)
        try:
            self.datadir = expanduser(config.get("General", "Datadir"))
        except ConfigParser.NoOptionError:
            print("Error: Please set a Datadir in ~/.cmddocsrc")
            exit(1)
        try:
            self.exclude = expanduser(config.get("General", "Excludedir"))
        except ConfigParser.NoOptionError:
            self.exclude = expanduser('.git/')
        try:
            self.default_commit_msg = config.get("General", "Default_Commit_Message")
        except ConfigParser.NoOptionError:
            self.default_commit_msg = "small changes"
        try:
            self.editor = config.get("General", "Editor")
        except ConfigParser.NoOptionError:
            if os.environ.get('EDITOR') is not None:
                self.editor = os.environ.get('EDITOR')
            else:
                print("Error: Could not find usable editor.")
                print("Please specify one in config or set EDITOR in your \
                OS Environment")
                exit(1)
        try:
            self.pager = config.get("General", "Pager")
        except ConfigParser.NoOptionError:
            if os.environ.get('PAGER') is not None:
                self.editor = os.environ.get('PAGER')
            else:
                print("Error: Could not find usable Pager.")
                print("Please specify one in config or set PAGER in your\
                OS Environment")
                exit(1)
        try:
            self.prompt = config.get("General", "Prompt")
        except ConfigParser.NoOptionError:
            self.prompt = "cmddocs>"
        try:
            self.promptcol = config.get("General", "Promptcolor")
        except ConfigParser.NoOptionError:
            self.promptcol = "37"
        try:
            self.intro = config.get("General", "Intro_Message")
        except ConfigParser.NoOptionError:
            self.intro = "cmddocs - press ? for help"
        try:
            self.mailfrom = config.get("General", "Mail")
        except ConfigParser.NoOptionError:
            self.mailfrom = "nobody"
        return

    def initialize_docs(self, docs):
        # Read or initialize git repository
        try:
            self.repo = git.Repo(self.datadir)
        except git.exc.NoSuchPathError:
            print("Error: Specified datadir %s does not exist" % self.datadir)
            exit(1)
        except git.exc.InvalidGitRepositoryError:
            self.repo = git.Repo.init(self.datadir)
            try:
                self.repo.git.add(".")
                self.repo.git.commit(m=" init")
            except git.exc.GitCommandError:
                pass
            print("Successfully created and initialized empty repo at %s" % self.datadir)
        # Change to datadir
        try:
            os.chdir(self.datadir)
            self.cwd = os.getcwd()
        except OSError:
            print("Error: Switching to Datadir %s not possible" % self.datadir)
            exit(1)

    ### list
    def do_list(self, dir):
        "Show files in current working dir"
        if not dir:
            dir = "."
        return list_articles(dir)

    # Aliases
    do_l = do_list
    do_ls = do_list

    def do_dirs(self, dir):
        "Show only directories in cwd"
        if not dir:
            dir = "."
        return list_directories(dir)

    do_d = do_dirs

    ### directories
    def do_cd(self,dir):
        "Change directory"
        cwd = change_directory(dir,self.datadir)

    def do_pwd(self,line):
        "Show current directory"
        print(os.path.relpath(os.getcwd(),self.datadir))

    ### edit
    def do_edit(self, article):
        """
        Edit or create new article.

        > edit databases/mongodb
        > edit intro
        """
        return edit_article(article, os.getcwd(), self.editor, self.repo, self.default_commit_msg)

    do_e = do_edit

    ### view
    def do_view(self, article):
        """
        View an article. Creates temporary file with converted markdown to
        ansi colored output. Opens your PAGER. (Only less supported atm)

        > view databases/mongodb
        > view intro
        """
        return view_article(article, os.getcwd(), self.pager)

    ### view
    def do_mail(self, article):
        """
        Mail an article to a friend

        > mail databases/mongodb
        > mail programming/r/loops
        """
        return mail_article(article, os.getcwd(), self.mailfrom)

    ### delete
    def do_delete(self, article):
        "Delete an article"
        delete_article(article, os.getcwd(),self.repo)

    do_rm = do_delete

    ### move
    def do_move(self, args):
        "Move an article"
        move_article(os.getcwd(),args,self.repo)

    do_mv = do_move

    ### search
    def do_search(self, keyword):
        "Search for keyword in current directory. Example: search mongodb"
        print(search_article(keyword,os.getcwd(), self.datadir,
            self.exclude))

    ### status
    def do_status(self, line):
        "Show git repo status of your docs"
        print(self.repo.git.status())

    def do_log(self, args):
        """
        Show git logs of your docs.

        Usage: log                      # default loglines: 10)
               log 20                   # show 20 loglines
               log 20 article           # show log for specific article
               log databases/mongodb 3  # same
        """
        show_log(args,self.repo)

    def do_diff(self, args):
        """
        Show git diffs (Gruesse von deiner Suessen!)

        Usage: log                      # default diff to last
        """
        show_diff(args,self.repo)


    ### undo / revert
    def do_undo(self, args):
        """
        You can revert your changes (use revert from git)

        Usage:
        > undo HEAD
        > undo 355f375

        Will ask for confirmation.
        """
        undo_change(args, self.repo)

    do_revert = do_undo

    ### exit
    def do_exit(self, line):
        "Exit cmddocs"
        return True

    def do_EOF(self, line):
        "Exit cmddocs"
        print("exit")
        return True

    ### completions
    complete_l = path_complete
    complete_ls = path_complete
    complete_list = path_complete

    complete_d = path_complete
    complete_dirs = path_complete

    complete_view = path_complete
    complete_cd = path_complete

    complete_e = path_complete
    complete_edit = path_complete

    complete_rm = path_complete
    complete_delete = path_complete

    complete_mail = path_complete
    complete_mv = path_complete
    complete_move = path_complete

    complete_log = path_complete

def ctrlc(sig, frame):
    print("\n")
    sys.exit(0)

signal.signal(signal.SIGINT, ctrlc)

def main():
    Cmddocs().cmdloop()

if __name__ == '__main__':
    main()
